package
{
	import flash.display.Graphics;
	import flash.geom.Point;
	
	public class EdgeBezierEx
	{
        private var baseLine:EdgeBezier;
        private var segment:uint = 150; //分割数
        private var poses1:Array; //tに対応した位置
        private var poses2:Array;
        private var thicknesses:Object;
        private var color:uint = 0;
        private var alpha:Number = 1;
        public var start_t:Number;
        public var end_t:Number;
        
		public function EdgeBezierEx(_p0:Point , _p1:Point , _p2:Point)
		{
            baseLine = new EdgeBezier(_p0 , _p1 , _p2);
            start_t = 0;
            end_t = 1;
            setThickness(1, 1);
		}

        private function getTangentPoint(isIn:Boolean, distance:Number, t:Number, isControl:Boolean = false):Point
        {
            var pt:Point = new Point();
            var deff:Point = baseLine.getDeffAt(t);
            var angle:Number = -Math.atan2(deff.x, deff.y);
            var disX:Number = Math.cos(angle) * distance;
            var disY:Number = Math.sin(angle) * distance;
            var basePt:Point;
            
            if(isControl == false)
            {
            	basePt = baseLine.getPosAt(t);
	            if(isIn)
	            {
	                pt.x = basePt.x + disX;
	                pt.y = basePt.y + disY;
	            }
	            else
	            {
	                pt.x = basePt.x - disX;
	                pt.y = basePt.y - disY;
	            }
            }
            else
            {
            	basePt = new Point(baseLine.p1x, baseLine.p1y);
            	var offset:Number = isIn ? 0.5 : - 0.5;
            	pt.x = basePt.x + disX * offset;
                pt.y = basePt.y + disY * offset;
            }

            return pt;
        }
        
        private function setPoses():void
        {
            poses1 = new Array(segment);
            poses2 = new Array(segment);
            
            var startPt1:Point = getTangentPoint(true, thicknesses.start / 2, 0);
            var ctrlPt1:Point = getTangentPoint(true, (thicknesses.start + thicknesses.end) / 2, 0.5, true);
            var endPt1:Point = getTangentPoint(true, thicknesses.end / 2, 1);
            var startPt2:Point = getTangentPoint(false, thicknesses.start / 2, 0);
            var ctrlPt2:Point = getTangentPoint(false, (thicknesses.start + thicknesses.end) / 2, 0.5, true);
            var endPt2:Point = getTangentPoint(false, thicknesses.end / 2, 1);
            var bezier1:EdgeBezier = new EdgeBezier(startPt1, ctrlPt1, endPt1);
            var bezier2:EdgeBezier = new EdgeBezier(startPt2, ctrlPt2, endPt2);
            var i:uint;
            var t:Number = 0;
            var dt:Number = 1 / segment;

            for(i = 0; i < segment; i++)
            {
               poses1[i] = bezier1.getPosAt(t);
               poses2[i] = bezier2.getPosAt(t);
               t += dt;
            }
        }
        
        public function setThickness(start:Number, end:Number):void
        {
            if(isNaN(end)){end = start}
            thicknesses = 
            {
                start:start, end:end
            }
            setPoses();
        }
        
        private function calcToSegment(target_t:Number):uint
        {
            var seg:uint = segment * target_t;
            return (seg >= segment) ? segment - 1 : seg;
        }
        
        public function draw(g:Graphics):void
        {
            var startSeg:uint = calcToSegment(start_t);
            var endSeg:uint = calcToSegment(end_t);
            var cnt:int = startSeg;
            g.clear();
            g.beginFill(color, alpha);
            g.moveTo(poses1[startSeg].x, poses1[startSeg].y);
            while(cnt <= endSeg)
            {
                g.lineTo(poses1[cnt].x, poses1[cnt].y);
                cnt++;
            }
            g.lineTo(poses2[endSeg].x, poses2[endSeg].y);
            
            cnt = endSeg;
            while(cnt >= startSeg)
            {
                g.lineTo(poses2[cnt].x, poses2[cnt].y);
                cnt--;
            }
            g.lineTo(poses1[startSeg].x, poses1[startSeg].y);
            g.endFill();
        }
        
        public function beginFill(color:uint = 0, alpha:Number = 1):void
        {
            this.color = color;
            this.alpha = alpha;
        }
	}
}